 /*
 * file      : src/chan.c
 * project   : tnt
 * with      : Gtk-3
 *
 * copyright : (C) 2012 by Claude BULIN
 *
 * TNT - INDIQUEZ ICI CE QUE FAIT LE PROGRAMME
 * GNU General Public License
 *
 * This file is part of TNT.
 *
 * TNT is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * at your option) any later version.
 *
 * TNT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TNT.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <glib.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../lib_dvbtv/lib.h"
#endif

#include "global.h"
#include "chan.h"
#include "configuser.h"


typedef struct {
	gchar	*Name;
	gint	Value;
} TABLE;

TABLE bandwidth_table[] =
{
	{ "BANDWIDTH_8_MHZ",	BANDWIDTH_8_MHZ },
	{ "BANDWIDTH_7_MHZ",	BANDWIDTH_7_MHZ },
	{ "BANDWIDTH_6_MHZ",	BANDWIDTH_6_MHZ },
	{ "BANDWIDTH_AUTO",		BANDWIDTH_AUTO },
	{ NULL, 0 }
};

TABLE fec_table[] =
{
	{ "FEC_NONE",	FEC_NONE },
	{ "FEC_1_2",	FEC_1_2 },
	{ "FEC_2_3",	FEC_2_3 },
	{ "FEC_3_4",	FEC_3_4 },
	{ "FEC_4_5",	FEC_4_5 },
	{ "FEC_5_6",	FEC_5_6 },
	{ "FEC_6_7",	FEC_6_7 },
	{ "FEC_7_8",	FEC_7_8 },
	{ "FEC_8_9",	FEC_8_9 },
	{ "FEC_AUTO",	FEC_AUTO },
	{ NULL, 0 }
};

TABLE modulation_table[] =
{
	{ "QPSK",		QPSK },
	{ "QAM_16",		QAM_16 },
	{ "QAM_32",		QAM_32 },
	{ "QAM_64",		QAM_64 },
	{ "QAM_128",	QAM_128 },
	{ "QAM_256",	QAM_256 },
	{ "QAM_AUTO",	QAM_AUTO },
	{ "8VSB",		VSB_8 },
	{ "VSB_16",		VSB_16 },
	{ NULL, 0 }
};

TABLE transmit_mode_table[] =
{
	{ "TRANSMISSION_MODE_2K",   TRANSMISSION_MODE_2K },
	{ "TRANSMISSION_MODE_8K",   TRANSMISSION_MODE_8K },
	{ "TRANSMISSION_MODE_AUTO", TRANSMISSION_MODE_AUTO },
	{ NULL, 0 }
};

TABLE guard_table[] =
{
	{ "GUARD_INTERVAL_1_32",	GUARD_INTERVAL_1_32 },
	{ "GUARD_INTERVAL_1_16",	GUARD_INTERVAL_1_16 },
	{ "GUARD_INTERVAL_1_8",		GUARD_INTERVAL_1_8 },
	{ "GUARD_INTERVAL_1_4",		GUARD_INTERVAL_1_4 },
	{ "GUARD_INTERVAL_AUTO",	GUARD_INTERVAL_AUTO },
	{ NULL, 0 }
};

TABLE hierarchy_table[] =
{
	{ "HIERARCHY_NONE", HIERARCHY_NONE },
	{ "HIERARCHY_1",    HIERARCHY_1 },
	{ "HIERARCHY_2",    HIERARCHY_2 },
	{ "HIERARCHY_4",    HIERARCHY_4 },
	{ "HIERARCHY_AUTO", HIERARCHY_AUTO },
	{ NULL, 0 }
};

TABLE inversion_table[] =
{
	{ "INVERSION_OFF",	INVERSION_OFF },
	{ "INVERSION_ON",	INVERSION_ON },
	{ "INVERSION_AUTO",	INVERSION_AUTO },
	{ NULL, 0 }
};

TABLE polarisation_table[] =
{
	{ "h",	0 },
	{ "v",	1 },
	{ NULL, 0 }
};
/*

	http://www.linuxtv.org/docs/dvbapi/DVB_Frontend_API.html
 	http://www.linuxtv.org/wiki/index.php/Dvbscan

0			1			2				3					4			5			6				7						8						9					10			11		12
Channel		Frequency	Inversion		Symbol				FEC name	FEC name	Constellation	Transmission			Guard					Hierarchy			Video		Audio	Service
name		(Hz)		mode			rate				HP			LP			mode			interval				information				PID					PID			ID

ABC HDTV 	226500000 	INVERSION_AUTO 	BANDWIDTH_7_MHZ 	FEC_3_4 	FEC_3_4 	QAM_64			TRANSMISSION_MODE_8K 	GUARD_INTERVAL_1_16		HIERARCHY_NONE 		2314 		0		592
TMC(SMR6)	562000000	INVERSION_AUTO	BANDWIDTH_8_MHZ		FEC_AUTO	FEC_AUTO	QAM_AUTO		TRANSMISSION_MODE_AUTO	GUARD_INTERVAL_AUTO		HIERARCHY_AUTO		620			630		1542
TF1(SMR6)	562000000	INVERSION_AUTO	BANDWIDTH_8_MHZ		FEC_AUTO	FEC_AUTO	QAM_AUTO		TRANSMISSION_MODE_AUTO	GUARD_INTERVAL_AUTO		HIERARCHY_AUTO		120			130		1537
TF6(SMR6)	562000000	INVERSION_AUTO	BANDWIDTH_8_MHZ		FEC_AUTO	FEC_AUTO	QAM_AUTO		TRANSMISSION_MODE_AUTO	GUARD_INTERVAL_AUTO		HIERARCHY_AUTO		520			530		1541
*/

/*
typedef struct {
	gchar	*NameChan;
	struct	dvb_frontend_parameters frontend_parameters;
	gint	Video_pid;
	gint	Audio_pid;
} CHANNEL;
*/
GList	*ListChannels = NULL;	// CHANNEL


gchar	**LarrbufLine = NULL;



gchar *chan_get_name( void )
{
	return( g_strdup( (gchar *)LarrbufLine[ 0 ] ));
}
gint chan_get_frequency( void )
{
	return( (gint)atoi( LarrbufLine[ 1 ] ));
}
fe_spectral_inversion_t chan_get_inversion( void )
{
	gint	Cpt;

	for( Cpt = 0; inversion_table[ Cpt ].Name; Cpt ++ ) {
		if( 0 == strcmp( inversion_table[ Cpt ].Name, LarrbufLine[ 2 ] )) {
			return( (fe_spectral_inversion_t) inversion_table[ Cpt ].Value );
		}
	}
	return( (fe_spectral_inversion_t)-1 );
}
fe_bandwidth_t chan_get_bandwidth( void )
{
	gint	Cpt;

	for( Cpt = 0; bandwidth_table[ Cpt ].Name; Cpt ++ ) {
		if( 0 == strcmp( bandwidth_table[ Cpt ].Name, LarrbufLine[ 3 ] )) {
			return( (fe_bandwidth_t)bandwidth_table[ Cpt ].Value );
		}
	}
	return( (fe_bandwidth_t)-1 );
}
fe_code_rate_t chan_get_fec_hp( void )
{
	gint	Cpt;

	for( Cpt = 0; fec_table[ Cpt ].Name; Cpt ++ ) {
		if( 0 == strcmp( fec_table[ Cpt ].Name, LarrbufLine[ 4 ] )) {
			return( (fe_code_rate_t)fec_table[ Cpt ].Value );
		}
	}
	return( (fe_code_rate_t)-1 );
}
fe_code_rate_t chan_get_fec_lp( void )
{
	gint	Cpt;

	for( Cpt = 0; fec_table[ Cpt ].Name; Cpt ++ ) {
		if( 0 == strcmp( fec_table[ Cpt ].Name, LarrbufLine[ 5 ] )) {
			return( (fe_code_rate_t)fec_table[ Cpt ].Value );
		}
	}
	return( (fe_code_rate_t)-1 );
}
fe_modulation_t chan_get_modulation( void )
{
	gint	Cpt;

	for( Cpt = 0; modulation_table[ Cpt ].Name; Cpt ++ ) {
		if( 0 == strcmp( modulation_table[ Cpt ].Name, LarrbufLine[ 6 ] )) {
			return( (fe_modulation_t)modulation_table[ Cpt ].Value );
		}
	}
	return( (fe_modulation_t)-1 );
}
fe_transmit_mode_t chan_get_transmit_mode( void )
{
	gint	Cpt;

	for( Cpt = 0; transmit_mode_table[ Cpt ].Name; Cpt ++ ) {
		if( 0 == strcmp( transmit_mode_table[ Cpt ].Name, LarrbufLine[ 7 ] )) {
			return( (fe_transmit_mode_t)transmit_mode_table[ Cpt ].Value );
		}
	}
	return( (fe_transmit_mode_t)-1 );
}
fe_guard_interval_t chan_get_guard_interval( void )
{
	gint	Cpt;

	for( Cpt = 0; guard_table[ Cpt ].Name; Cpt ++ ) {
		if( 0 == strcmp( guard_table[ Cpt ].Name, LarrbufLine[ 8 ] )) {
			return( (fe_guard_interval_t)guard_table[ Cpt ].Value );
		}
	}
	return( (fe_guard_interval_t)-1 );
}
fe_hierarchy_t chan_get_hierarchy( void )
{
	gint	Cpt;

	for( Cpt = 0; hierarchy_table[ Cpt ].Name; Cpt ++ ) {
		if( 0 == strcmp( hierarchy_table[ Cpt ].Name, LarrbufLine[ 9 ] )) {
			return( (fe_hierarchy_t)hierarchy_table[ Cpt ].Value );
		}
	}
	return( (fe_hierarchy_t)-1 );
}
gint chan_get_Video_pid( void )
{
	return( (gint)atoi( LarrbufLine[ 10 ] ));
}
gint chan_get_Audio_pid( void )
{
	return( (gint)atoi( LarrbufLine[ 11 ] ));
}
gint chan_get_Text_pid( void )
{
	return( (gint)atoi( LarrbufLine[ 12 ] ));
}


//
//   /home/cat/channels.conf
//
gboolean chan_read( void )
{
	FILE	*fp;
#define MAX_CARS 255
	gchar	Buffer [ MAX_CARS +1 ];
	gchar	*Ptr = NULL;
	CHANNEL	*Channel = NULL;
	gint	nbr_separator = 0;

	// open channel.conf
	// 	if( NULL == (fp = fopen( "/home/cat/channels.conf", "r" )) ) return( FALSE );
	if( NULL == (fp = fopen( Config.PathName_channel_conf, "r" )) ) return( FALSE );

	// read channel.conf
	while (fgets (Buffer, MAX_CARS, fp) != NULL) {

		// test bad line
		for( nbr_separator = 0, Ptr = Buffer; *Ptr; Ptr++ )
			if( *Ptr == ':' ) nbr_separator ++;
		if( nbr_separator != 12 ) continue;

		// remove carriage-return
		for( Ptr = Buffer; *Ptr; Ptr++ )
			if( *Ptr == '\n' ) {
				*Ptr = '\0';
				break;
			}

		// split line
		LarrbufLine = g_strsplit( Buffer, ":", 0 );
		if( LarrbufLine[0] == NULL ) {
			// remove split line
			g_strfreev( LarrbufLine );
			LarrbufLine = NULL;
			break;
		}

		// allocate struct
		Channel = (CHANNEL *)g_malloc0( sizeof(CHANNEL) );

		// set datas
		Channel->NameChan											= chan_get_name();
		Channel->frontend_parameters.frequency						= chan_get_frequency();
		Channel->frontend_parameters.inversion						= chan_get_inversion();
		Channel->frontend_parameters.u.ofdm.bandwidth				= chan_get_bandwidth();
		Channel->frontend_parameters.u.ofdm.code_rate_HP			= chan_get_fec_hp();
		Channel->frontend_parameters.u.ofdm.code_rate_LP			= chan_get_fec_lp();
		Channel->frontend_parameters.u.ofdm.constellation			= chan_get_modulation();
		Channel->frontend_parameters.u.ofdm.transmission_mode		= chan_get_transmit_mode();
		Channel->frontend_parameters.u.ofdm.guard_interval			= chan_get_guard_interval();
		Channel->frontend_parameters.u.ofdm.hierarchy_information	= chan_get_hierarchy();
		Channel->Video_pid											= chan_get_Video_pid();
		Channel->Audio_pid											= chan_get_Audio_pid();
		Channel->Text_pid											= chan_get_Text_pid();

		// to treeview
		add_to_treeview( FALSE,  Channel->NameChan );

		// append list
		ListChannels = g_list_append( ListChannels, Channel );

		// remove split line
		g_strfreev( LarrbufLine );
		LarrbufLine = NULL;
	}

	// close channel.conf
	fclose( fp );

	return( TRUE );
}

void chan_remove_list( void )
{
	CHANNEL	*Channel = NULL;
	GList	*List = NULL;

	if( NULL != (List = g_list_first( ListChannels ))) {
		while( List ) {
			if( NULL != (Channel = (CHANNEL *)List->data) ) {
				if( Channel->NameChan != NULL) {
					g_free( Channel->NameChan );
					Channel->NameChan = NULL;
				}
				g_free( Channel );
				List->data = Channel = NULL;
			}
			List = g_list_next( List );
		}
		g_list_free( ListChannels );
		ListChannels = NULL;
	}
}

CHANNEL *chan_get_struct( gchar *p_NameChan )
{
	CHANNEL	*Channel = NULL;
	GList	*List = NULL;

	if( NULL != (List = g_list_first( ListChannels ))) {
		while( List ) {
			if( NULL != (Channel = (CHANNEL *)List->data) ) {
				if( 0 == (strcmp(Channel->NameChan, p_NameChan))) {
					return( Channel );
				}
			}
			List = g_list_next( List );
		}
	}
	return( NULL );
}

void chan_get_param( gchar *p_NameChan, struct dvb_frontend_parameters *p_param )
{
	CHANNEL		*Channel = NULL;

	Channel = chan_get_struct( p_NameChan );
	p_param->frequency						= Channel->frontend_parameters.frequency;
	p_param->inversion						= Channel->frontend_parameters.inversion;
	p_param->u.ofdm.bandwidth				= Channel->frontend_parameters.u.ofdm.bandwidth;
	p_param->u.ofdm.code_rate_HP			= Channel->frontend_parameters.u.ofdm.code_rate_HP;
	p_param->u.ofdm.code_rate_LP			= Channel->frontend_parameters.u.ofdm.code_rate_LP;
	p_param->u.ofdm.constellation			= Channel->frontend_parameters.u.ofdm.constellation;
	p_param->u.ofdm.transmission_mode		= Channel->frontend_parameters.u.ofdm.transmission_mode;
	p_param->u.ofdm.guard_interval			= Channel->frontend_parameters.u.ofdm.guard_interval;
	p_param->u.ofdm.hierarchy_information	= Channel->frontend_parameters.u.ofdm.hierarchy_information;

	if( TRUE == OptionsCommandLine.BoolVerboseMode ) {
		g_print( "\t---\n" );
		g_print("\tNameChan                     = %s\n"
				"\tfrequency                    = %d\n"
				"\tinversion                    = %d\n"
				"\tu.ofdm.bandwidth             = %d\n"
				"\tu.ofdm.code_rate_HP          = %d\n"
				"\tu.ofdm.code_rate_LP          = %d\n"
				"\tu.ofdm.constellation         = %d\n"
				"\tu.ofdm.transmission_mode     = %d\n"
				"\tu.ofdm.guard_interval        = %d\n"
				"\tu.ofdm.hierarchy_information = %d\n"
				"\tVideo_pid                    = %d\n"
				"\tAudio_pid                    = %d\n",
				p_NameChan,
				p_param->frequency,
				p_param->inversion,
				p_param->u.ofdm.bandwidth,
				p_param->u.ofdm.code_rate_HP,
				p_param->u.ofdm.code_rate_LP,
				p_param->u.ofdm.constellation,
				p_param->u.ofdm.transmission_mode,
				p_param->u.ofdm.guard_interval,
				p_param->u.ofdm.hierarchy_information,
				Channel->Video_pid,
				Channel->Audio_pid );
	}
}

