 /*
 *  file      : configuser_new.c
 *  project   : xcfa
 *  with      : Gtk-2
 *
 *  copyright : (C) 2003 - 2013 by Claude Bulin
 *
 *  xcfa - GTK+ implementation of the GNU shell command
 *  GNU General Public License
 *
 *  This file is part of XCFA.
 *
 *  XCFA is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  at your option) any later version.
 *
 *  XCFA is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with XCFA.  If not, see <http://www.gnu.org/licenses/>.
 */
 /*
 * New Developpment File:
 * Fri, 15 Jul 2011 22:45:07 +0200
 */


#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../lib_dvbtv/lib.h"
#endif

#include "configuser.h"
#include "global.h"



typedef struct {
	gchar	**Larrbuf;
} CONF;

CONF	Conf = { NULL };

VAR_CONFIG ConfigSaveToRest;	// Pour les restitutions par defaut
VAR_CONFIG Config = {
	FALSE,		// BoolConfigOk
	1,			// gulong	UsesOfXcfa;	// 0 to G_MAXULONG ( 18446744073709551615 )
	0,			// WinPos_X
	0,			// WinPos_Y
	800,		// WinWidth
	520,		// WinHeight

	NULL		// PathName_channel_conf
};



// TRUE SI p_dest EST DANS p_src SINON FALSE
//
gboolean config_new_strcmp (gchar *p_src, gchar *p_dest)
{
	gint	len_aiguille = strlen (p_dest);
	gint	len_meule;
	gint	len;
	gchar	*PtrBeginSrc = NULL;
	gchar	*PtrEndSrc = NULL;
	gchar	*PtrSrc = NULL;
	gchar	*PtrDest = p_dest;

	for (PtrBeginSrc = p_src; *PtrBeginSrc == ' ' || *PtrBeginSrc == '\t'; PtrBeginSrc ++);
	for (PtrEndSrc = PtrBeginSrc; *PtrEndSrc != ' ' && *PtrEndSrc != '\t'; PtrEndSrc ++);
	len_meule = PtrEndSrc - PtrBeginSrc;

	if (len_meule != len_aiguille) return (FALSE);

	len = 0;
	for (PtrSrc = PtrBeginSrc; PtrSrc < PtrEndSrc; PtrSrc ++, PtrDest ++) {
		if (*PtrSrc != *PtrDest) return (FALSE);;
		len ++;
	}

	return (len_meule == len ? TRUE : FALSE);
}
//
//
void config_print_data (gchar *p_str)
{
	gint	len = strlen( p_str );
	g_print( "%s", p_str );
	while( len < 33 ) {
		g_print( " " );
		len ++;
	}
	g_print( "= " );
}
// RETOURNE UNE DEFINITION DU FICHIER DE CONFIGURATION
//
gchar *config_get_target( gchar *p_buf )
{
	gchar	*Str = NULL;
	gchar	*Ptr = NULL;

	if( NULL != ( Ptr = strchr( p_buf, '=' ))) {
		Ptr ++;
		while( *Ptr && (*Ptr != ' ' && *Ptr != '\t' )) Ptr ++;
		Ptr ++;
		Str = g_strdup( Ptr );
		if( NULL != ( Ptr = strchr( Str, '\r' ))) *Ptr = '\0';
		if( NULL != ( Ptr = strchr( Str, '\n' ))) *Ptr = '\0';
	}
	return( (gchar *)Str );
}
//
//
gboolean config_get_bool( gchar *value )
{
	gchar *ptr = value;

	while( *ptr ) {
		*ptr = toupper( *ptr );
		ptr++;
	}
	return ( !strncmp( value, "TRUE", strlen( "TRUE" )) ? TRUE : FALSE );
}
// LECTURE BOOLEAN
//
gboolean config_read_value_bool( gchar *p_Search, gboolean *p_ValueBool )
{
	gchar	*Ptr = NULL;
	gint	Cpt;

	for( Cpt = 0; Conf.Larrbuf[ Cpt ]; Cpt++ ) {

		// Si commentaire ou ligne vide alors ligne suivante
		if( *Conf.Larrbuf[ Cpt ] == '#' || *Conf.Larrbuf[ Cpt ] == '\0') continue;

		if( TRUE == config_new_strcmp( Conf.Larrbuf[ Cpt ], p_Search )) {
			if( NULL != ( Ptr = config_get_target( Conf.Larrbuf[ Cpt ] ))) {
				*p_ValueBool = config_get_bool( Ptr );
				g_free( Ptr );
				Ptr = NULL;
				if( TRUE == OptionsCommandLine.BoolVerboseMode ) {
					config_print_data( p_Search );
					g_print( "%s\n", *p_ValueBool ? "TRUE" : "FALSE" );
				}
				*Conf.Larrbuf[ Cpt ] = '\0';
				return( TRUE );
			}
		}
	}
	return( FALSE );
}
// LECTURE NUMERIQUE INT
//
gboolean config_read_value_atoi( gchar *p_Search, gint *p_ValueAtoi, gint p_Min, gint p_Max )
{
	gchar	*Ptr = NULL;
	gint	Cpt;

	for( Cpt = 0; Conf.Larrbuf[ Cpt ]; Cpt++ ) {

		// Si commentaire ou ligne vide alors ligne suivante
		if( *Conf.Larrbuf[ Cpt ] == '#' || *Conf.Larrbuf[ Cpt ] == '\0') continue;

		if( TRUE == config_new_strcmp( Conf.Larrbuf[ Cpt ], p_Search )) {
			if( NULL != ( Ptr = config_get_target( Conf.Larrbuf[ Cpt ] ))) {
				*p_ValueAtoi = atoi( Ptr );
				g_free( Ptr );
				Ptr = NULL;
				if( p_Min > -1 && *p_ValueAtoi < p_Min ) *p_ValueAtoi = p_Min;
				if( p_Max > -1 && *p_ValueAtoi > p_Max ) *p_ValueAtoi = p_Max;
				if( TRUE == OptionsCommandLine.BoolVerboseMode ) {
					config_print_data( p_Search );
					g_print( "%d\n", *p_ValueAtoi ? *p_ValueAtoi : 0 );
				}
				*Conf.Larrbuf[ Cpt ] = '\0';
				return( TRUE );
			}
		}
	}
	return( FALSE );
}
// LECTURE NUMERIQUE INT
//
gboolean config_read_value_atol( gchar *p_Search, gulong *p_ValueAtoi, gint p_Min, gint p_Max )
{
	gchar	*Ptr = NULL;
	gint	Cpt;

	for( Cpt = 0; Conf.Larrbuf[ Cpt ]; Cpt++ ) {

		// Si commentaire ou ligne vide alors ligne suivante
		if( *Conf.Larrbuf[ Cpt ] == '#' || *Conf.Larrbuf[ Cpt ] == '\0') continue;

		if( TRUE == config_new_strcmp( Conf.Larrbuf[ Cpt ], p_Search )) {
			if( NULL != ( Ptr = config_get_target( Conf.Larrbuf[ Cpt ] ))) {
				*p_ValueAtoi = atol( Ptr );
				g_free( Ptr );
				Ptr = NULL;
				if( p_Min > -1 && *p_ValueAtoi < p_Min ) *p_ValueAtoi = p_Min;
				if( p_Max > -1 && *p_ValueAtoi > p_Max ) *p_ValueAtoi = p_Max;
				if( TRUE == OptionsCommandLine.BoolVerboseMode ) {
					config_print_data( p_Search );
					g_print( "%lu / %lu  ;-)\n", *p_ValueAtoi ? *p_ValueAtoi : 0, G_MAXULONG );
				}
				*Conf.Larrbuf[ Cpt ] = '\0';
				return( TRUE );
			}
		}
	}
	return( FALSE );
}
// LECTURE PATHNAME
//
gboolean config_read_value_path( gchar *p_Search, gchar **p_ValuePath, gchar *p_default )
{
	gchar	*Ptr = NULL;
	gint	Cpt;

	for( Cpt = 0; Conf.Larrbuf[ Cpt ]; Cpt++ ) {

		// Si commentaire ou ligne vide alors ligne suivante
		if( *Conf.Larrbuf[ Cpt ] == '#' || *Conf.Larrbuf[ Cpt ] == '\0') continue;

		if( TRUE == config_new_strcmp( Conf.Larrbuf[ Cpt ], p_Search )) {
			if( NULL != ( Ptr = config_get_target( Conf.Larrbuf[ Cpt ] ))) {
				if( TRUE == g_file_test( Ptr, G_FILE_TEST_IS_DIR )) {
					*p_ValuePath = Ptr;
				} else {
					if( NULL == p_default )	*p_ValuePath = g_strdup( g_getenv( "HOME" ));
					else			*p_ValuePath = g_strdup( p_default );
					g_free( Ptr );
					Ptr = NULL;
				}
			}
			else {
				if( NULL == p_default )	*p_ValuePath = g_strdup( g_getenv( "HOME" ));
				else			*p_ValuePath = g_strdup( p_default );
			}
			if( TRUE == OptionsCommandLine.BoolVerboseMode ) {
				config_print_data( p_Search );
				g_print( "%s\n", *p_ValuePath ? *p_ValuePath : "" );
			}
			*Conf.Larrbuf[ Cpt ] = '\0';
			return( TRUE );
		}
	}
	return( FALSE );
}
//
//
gboolean config_read_value_str( gchar *p_Search, gchar **p_ValueStr )
{
	gchar	*Ptr = NULL;
	gint	Cpt;

	for( Cpt = 0; Conf.Larrbuf[ Cpt ]; Cpt++ ) {

		// Si commentaire ou ligne vide alors ligne suivante
		if( *Conf.Larrbuf[ Cpt ] == '#' || *Conf.Larrbuf[ Cpt ] == '\0') continue;

		if( TRUE == config_new_strcmp( Conf.Larrbuf[ Cpt ], p_Search )) {
			if( NULL != ( Ptr = config_get_target( Conf.Larrbuf[ Cpt ]))) {
				*p_ValueStr = g_strdup( Ptr );
				g_free( Ptr );
				Ptr = NULL;
				if( TRUE == OptionsCommandLine.BoolVerboseMode ) {
					config_print_data( p_Search );
					g_print( "%s\n", *p_ValueStr ? *p_ValueStr : "" );
				}
				*Conf.Larrbuf[ Cpt ] = '\0';
				return( TRUE );
			}
		}
	}
	return( FALSE );
}
//
// CALL:
// 	config_read_value_tab( "HOURS_MONDAY", Config.HoursMonday, 4 );
//
gboolean config_read_value_tab( gchar *p_Search, gint p_ValueTab[], gint p_max_tab )
{
	gchar	*Ptr = NULL;
	gint	Cpt;

	// IF NOT INIT IN TAB:
	// 	*(p_ValueTab +0) = 0;
	// 	*(p_ValueTab +1) = 0;
	// 	*(p_ValueTab +2) = 0;
	// 	*(p_ValueTab +3) = 0;

	for( Cpt = 0; Conf.Larrbuf[ Cpt ]; Cpt++ ) {

		// Si commentaire ou ligne vide alors ligne suivante
		if( *Conf.Larrbuf[ Cpt ] == '#' || *Conf.Larrbuf[ Cpt ] == '\0') continue;

		if( TRUE == config_new_strcmp( Conf.Larrbuf[ Cpt ], p_Search )) {

			if( TRUE == OptionsCommandLine.BoolVerboseMode )
				config_print_data( p_Search );

			if( NULL != (Ptr = strchr( Conf.Larrbuf[ Cpt ], '=' ))) {

				gchar	**LarrbufLine = NULL;
				gint	Indice;

				// POINTEUR SUR LA PREMIERE OCCURANCE
				Ptr ++;
				while( *Ptr == ' ' || *Ptr == '\t' ) Ptr ++;

				// *Ptr IS VALID ?
				if( *Ptr != '\0' ) {

					// HOURS_MONDAY                  = 435 690 720 975
					// HOURS_SATURDAY                = 0 0 0 0
					LarrbufLine = g_strsplit( Ptr, " ", 0 );

					// INDICE: 0 .. p_max_tab
					for( Indice = 0; LarrbufLine[ Indice ] && Indice < p_max_tab; Indice ++ ) {

						*(p_ValueTab +Indice) = (gint)atoi( LarrbufLine[ Indice ] );

						if( TRUE == OptionsCommandLine.BoolVerboseMode ) {
							g_print ("%04d (%02d:%02d)  ",
								*(p_ValueTab +Indice),
								*(p_ValueTab +Indice) / 60,
								*(p_ValueTab +Indice) % 60
								);
						}
					}

					g_strfreev( LarrbufLine );
					LarrbufLine = NULL;
				}
			}

			if( TRUE == OptionsCommandLine.BoolVerboseMode )
				g_print( "\n" );

			*Conf.Larrbuf[ Cpt ] = '\0';
			return( TRUE );
		}
	}
	return( FALSE );
}
//
//
void config_close_conf_file( void )
{
	if( NULL != Conf.Larrbuf ) {
		g_strfreev( Conf.Larrbuf );
		Conf.Larrbuf = NULL;
	}
}
//
//
gboolean config_open_conf_file( gchar *p_PathNameConfig )
{
	size_t 		SizeFile;
	gchar		*Buffer = NULL;
	FILE		*fp = NULL;

	config_close_conf_file();
	if(( SizeFile = libutils_get_size_file( p_PathNameConfig )) > 0 ) {
		if( NULL != ( fp = fopen( p_PathNameConfig, "r" ))) {
			Buffer = (gchar *)g_malloc0( sizeof(gchar) * ( SizeFile +10 ));
			SizeFile = fread( Buffer, SizeFile, 1, fp );
			fclose( fp );
			Conf.Larrbuf = g_strsplit( Buffer, "\n", 0 );
			g_free( Buffer );
			Buffer = NULL;
			return( TRUE );
		}
	}
	return( FALSE );
}
//
//
void config_verif_path (gchar **p_ValuePath, gchar *p_default)
{
	if( NULL == *p_ValuePath ) {
		if( NULL == p_default )	*p_ValuePath = g_strdup( g_getenv( "HOME" ));
		else			*p_ValuePath = g_strdup( p_default );
	}
}
//
//
// LECTURE DU FICHIER DE CONFIGURATION
//
void config_read( void )
{
	// gchar	*PathNameConfig = config_get_pathname ();	// HOME /.config/xcfa/xcfa.conf
	gchar	*PathNameConfig = xdg_search_config_xdg();		// HOME /.config/xcfa/xcfa.conf

	// Pour les restitutions par defaut
	g_memmove (&ConfigSaveToRest, &Config, sizeof (VAR_CONFIG));

	if( TRUE == OptionsCommandLine.BoolVerboseMode ) {
		g_print ("\n------------------------------------------------------------------------\n" );
		g_print ("%s\n", PathNameConfig);
		g_print ("------------------------------------------------------------------------\n" );
	}

	// IF NOT EXIST CONFIG-FILE
	if( FALSE == libutils_test_file_exist( PathNameConfig )) {
		// GET PATHNAME
		gchar	*StrDir = g_strdup( PathNameConfig );
		// GET PATH
		gchar	*Ptr = strrchr( StrDir, '/' );

		// NOW StrDir IS PATH
		Ptr ++;
		*Ptr = '\0';
		// CREATION DU DOSSIER
		g_mkdir_with_parents (StrDir, 0700);
		// FREE MEM
		g_free( StrDir );	StrDir = Ptr = NULL;
		// CREATE CONFIG-FILE
		config_save ();
	}

	if( TRUE == config_open_conf_file( PathNameConfig )) {

		// READ ATOL
		//
		config_read_value_atol( "USES_OF",					&Config.UsesOf,       		 0,	-1 );
		if( Config.UsesOf < G_MAXULONG )
			Config.UsesOf ++;
		else	Config.UsesOf = 0;

		// READ ATOI
		//
		config_read_value_atoi( "WIN_POS_X",				&Config.WinPos_X,       		 0,	-1 );
		config_read_value_atoi( "WIN_POS_Y",				&Config.WinPos_Y,       		 0,	-1 );
		config_read_value_atoi( "WIN_WIDTH",				&Config.WinWidth,        		-1,	-1 );
		config_read_value_atoi( "WIN_HEIGHT",				&Config.WinHeight,       		-1,	-1 );

		// READ BOOL
		//

		// READ PATH
		//
		// config_read_value_path( "PATHNAME_CHANNELS_CONF",	&Config.PathName_channel_conf,		NULL );

		// READ STRING
		//
		config_read_value_str( "PATHNAME_CHANNELS_CONF",	&Config.PathName_channel_conf );

		config_close_conf_file();

		config_verif_path( &Config.PathName_channel_conf,	(gchar *)g_getenv( "HOME" ) );
	}

	if( NULL != PathNameConfig ) {
		g_free( PathNameConfig );
		PathNameConfig = NULL;
	}
}
//  SAUVEGARDE DU FICHIER DE CONFIGURATION
//
void config_save( void )
{
	// gchar	*PathNameConfig = config_get_pathname();	// HOME /.config/xcfa/xcfa.conf
	gchar	*PathNameConfig = xdg_search_config_xdg();		// HOME /.config/xcfa/xcfa.conf
	FILE	*fp;
	// gchar	*Ptr = NULL;

	if( NULL != ( fp = fopen( PathNameConfig, "w" ))) {
		fprintf( fp,   "############################################################################\n" );
		fprintf( fp,   "## Fichier de configuration pour : %s\n", PACKAGE_STRING);
		fprintf( fp,   "##--------------------------------------------------------------------------\n" );
		fprintf( fp,   "## File of configuration to : %s\n", PACKAGE_STRING);
		fprintf( fp,   "############################################################################\n" );
		fprintf( fp, "\n" );
		fprintf( fp, "USES_OF                             = %lu\n",Config.UsesOf );
		fprintf( fp, "WIN_POS_X                           = %d\n", Config.WinPos_X );
		fprintf( fp, "WIN_POS_Y                           = %d\n", Config.WinPos_Y );
		fprintf( fp, "WIN_WIDTH                           = %d\n", Config.WinWidth );
		fprintf( fp, "WIN_HEIGHT                          = %d\n", Config.WinHeight );

		fprintf( fp, "PATHNAME_CHANNELS_CONF              = %s\n", Config.PathName_channel_conf ? Config.PathName_channel_conf : "" );

		// TODO: RESERVED
		// g_sprintf( String, "%08lx", ret );
		// long atol( const char *nptr );
		// La fonction atol() convertit le début de la chaîne pointée par nptr en entier de type long
		// The atol() function converts the initial portion of the string pointed to by nptr to long

		fprintf( fp, "\n" );
		fclose( fp );
	}
	if( NULL != PathNameConfig ) {
		g_free( PathNameConfig );
		PathNameConfig = NULL;
	}
}
// LIBERATION MEMOIRE
//
void config_remove( void )
{
	g_free( Config.PathName_channel_conf );		Config.PathName_channel_conf = NULL;
}



