 /*
 * file      : src/stream_tv.c
 * project   : tnt
 * with      : Gtk-3
 *
 * copyright : (C) 2012 by Claude BULIN
 *
 * TNT - INDIQUEZ ICI CE QUE FAIT LE PROGRAMME
 * GNU General Public License
 *
 * This file is part of TNT.
 *
 * TNT is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * at your option) any later version.
 *
 * TNT is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TNT.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include <glib.h>
#include <glib/gstdio.h>

#include <gtk/gtkx.h>

#include <pthread.h>
#include <sys/types.h>
#include <signal.h>
#include <sys/wait.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>	// PROCESSUS NON-BLOQUANT
#include <poll.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <stdio.h>
#include <errno.h>

#include <linux/dvb/frontend.h>
#include <linux/dvb/dmx.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../lib_dvbtv/lib.h"
#endif

#include "global.h"
#include "chan.h"
#include "stream.h"
#include "win_info.h"


#define TMP_FIFO_AVI "/tmp/tnt_fifo"

#define FD_READ		DataTv.in_fd[  0 ]
#define FD_WRITE	DataTv.out_fd[ 1 ]

#define	MAX_ARG_CONV_TV 30


typedef struct {

	GtkWidget	*AdrHboxVideo;			// Adresse du widget pour la video
	guint32		 outputHandle;			// Le PID de la video pour mplayer
	GtkWidget	*socket;				// Socket pour la video
	GtkWidget	*event;					// La boite pour la video
	gint		 width;					// widget's width
	gint		 height;				// widget's height
	gint		descripteur_fifo_avi;	//
	gboolean	bool_go_tv;				//
	pid_t		NumFork;				//
	guint		NumTimeout;				//
	int			out_fd[ 2 ];			// out mplayer
	int			in_fd[  2 ];			// in mplayer
	guint		buf[ 1504 ];			// buffer dvb
	gchar		*ArgConv [ MAX_ARG_CONV_TV + 10 ];
	gchar		*RepFifo;				// Directory fifo

} DATA_TV;

DATA_TV		DataTv = {
		NULL,		// GtkWidget	*AdrHboxVideo
		0,			// guint32		 outputHandle
		NULL,		// GtkWidget	*socket
		NULL,		// GtkWidget	*event
		0,			// gint			width
		0,			// gint			height
		-1,			// gint			descripteur_fifo_avi
		FALSE,		// gboolean		bool_go_tv
		0,			// pid_t		NumFork
		0,			// guint		NumTimeout
		{0,0},		// int			out_fd[ 2 ]
		{0,0},		// int			in_fd[  2 ]
		{0},		// guint		buf[ 1504 ]
		{0},		// gchar		*ArgConv [ MAX_ARG_CONV_TV + 20 ]
		NULL		// gchar		*RepFifo;

};

//
// http://stackoverflow.com/questions/280571/how-to-control-popen-stdin-stdout-stderr-redirection
// http://jineshkj.wordpress.com/2006/12/22/how-to-capture-stdin-stdout-and-stderr-of-child-program/
//
void StreamTv_exec( char **args )
{
	char	**ptr = (char **)args;

	pipe( DataTv.out_fd );			// Where the parent is going to write to
	pipe( DataTv.in_fd );			// From where parent is going to read
	if( ! ( DataTv.NumFork = fork()) )
	{
		g_print("getpid()       = %d\n", getpid() );
		g_print("DataTv.NumFork = %d\n", DataTv.NumFork );

		close( STDOUT_FILENO );
		close( STDIN_FILENO );
		dup2( DataTv.out_fd[0], STDIN_FILENO );
		dup2( DataTv.in_fd[1], STDOUT_FILENO );
		close( DataTv.out_fd[0] );	// Not required for the child
		close( DataTv.out_fd[1] );
		close( DataTv.in_fd[0] );
		close( DataTv.in_fd[1] );
		execvp ((char *)*(ptr+0), ptr);
	}
	close( DataTv.out_fd[0] );		// These are being used by the child
	close( DataTv.in_fd[1] );
}

gint StreamTv_reset_struct( void )
{
	gint	i;

	for (i = 0; i < MAX_ARG_CONV_TV; i++)
		DataTv.ArgConv [ i ] = NULL;
	return( 0 );
}
gint StreamTv_remove( void )
{
	gint	pos = 0;

	for (pos = 0; DataTv.ArgConv [ pos ] != NULL; pos ++) {
		g_free (DataTv.ArgConv [ pos ]);
		DataTv.ArgConv [ pos ] = NULL;
	}
	return (0);
}
void StreamTv_make_args( gchar *p_text, ... )
{
	gint	pos;
	va_list	VaList;
	gchar	*Ptr = NULL;

	pos = StreamTv_reset_struct();

	DataTv.ArgConv [ pos++ ] = g_strdup( p_text );
	va_start( VaList, p_text );
	while( NULL != ( Ptr = va_arg( VaList, gchar * ))) {
		DataTv.ArgConv [ pos++ ] = g_strdup( Ptr );
	}
	va_end( VaList );
	DataTv.ArgConv [ pos++ ] = NULL;

	if( TRUE == OptionsCommandLine.BoolVerboseMode ) {
		g_print ("\n!-----------------------------------------------------------!\n! ");
		for (pos = 0; DataTv.ArgConv[ pos ] != NULL; pos ++) {
			g_print ("%s ", DataTv.ArgConv[ pos ]);
		}
		g_print ("\n!-----------------------------------------------------------!\n");
		for (pos = 0; DataTv.ArgConv[ pos ] != NULL; pos ++) {
			g_print ("! ArgConv[ %02d ] = %s\n", pos, DataTv.ArgConv[ pos ]);
		}
		g_print ("!-----------------------------------------------------------!\n\n");
	}
}




void change_size( GtkWidget *widget, GtkAllocation *allocation, gpointer user_data)
{
	DATA_TV *p = (DATA_TV *) user_data;

	p->width  = allocation->width;
	p->height = allocation->height;
}
void on_box_video_realize( GtkWidget *widget, gpointer user_data )
{
	g_signal_connect(( gpointer)widget, "size-allocate", G_CALLBACK(change_size), &DataTv );

	DataTv.AdrHboxVideo = widget;
	// Creation de la boite d'evenements
	DataTv.event = gtk_event_box_new();
	// Creation du socket
	DataTv.socket = gtk_socket_new();
	// Integration
	gtk_box_pack_start( GTK_BOX(widget), DataTv.event, TRUE, TRUE, 2 );
	gtk_container_add( GTK_CONTAINER(DataTv.event), DataTv.socket );
	gtk_event_box_set_above_child( GTK_EVENT_BOX(DataTv.event), TRUE );
	// Prise en compte des evenements
	gtk_widget_show_all( DataTv.event );
	// Recuperation pour [ -wid ] du numero du socket
	DataTv.outputHandle = gtk_socket_get_id( GTK_SOCKET(DataTv.socket) );
	// Prise en compte du socket
	gtk_widget_show( DataTv.socket );
}



void StreamTv_mplayer_remove_fifo( void )
{
	if( NULL != DataTv.RepFifo && TRUE == g_file_test( DataTv.RepFifo, G_FILE_TEST_EXISTS )) {
		g_unlink( DataTv.RepFifo );
		g_free( DataTv.RepFifo );
		DataTv.RepFifo = NULL;
	}
}
void StreamTv_mplayer_create_fifo( void )
{
	StreamTv_mplayer_remove_fifo();
	// mkfifo( TMP_FIFO_AVI, 0644 );
	DataTv.RepFifo = libutils_create_fifo( TMP_FIFO_AVI );
	DataTv.descripteur_fifo_avi = open( DataTv.RepFifo, O_RDWR | O_NONBLOCK );		// | O_NONBLOCK
}



gboolean StreamTv_switch_to_record( int *p_demux_Video_pid, int *p_demux_Audio_pid, CHANNEL *p_Channel )
{
	struct dmx_pes_filter_params pesFilterParams;

	if( *p_demux_Video_pid < 0 ) {
		if( (*p_demux_Video_pid = open( adapter.demux, O_RDWR | O_NONBLOCK )) < 0 ) {
				perror( "DEMUX-VIDEO-PID DEVICE: " );
				return( FALSE );
		}
	}
	if( *p_demux_Audio_pid < 0 ) {
		if( (*p_demux_Audio_pid = open( adapter.demux, O_RDWR | O_NONBLOCK )) < 0 ) {
				perror( "DEMUX-AUDIO-PID DEVICE: " );
				close( *p_demux_Video_pid );
				return( FALSE );
		}
	}
	pesFilterParams.pid      = p_Channel->Video_pid;
	pesFilterParams.input    = DMX_IN_FRONTEND;
	pesFilterParams.output   = DMX_OUT_TS_TAP;
	pesFilterParams.pes_type = DMX_PES_VIDEO;		// DMX_PES_OTHER  DMX_PES_SUBTITLE
	pesFilterParams.flags    = DMX_IMMEDIATE_START;
	if( ioctl( *p_demux_Video_pid, DMX_SET_PES_FILTER, &pesFilterParams ) < 0 ) {
			perror( "DEMUX-VIDEO-PID DEVICE" );
			close( *p_demux_Video_pid );
			close( *p_demux_Audio_pid );
			return( FALSE );
	}

	pesFilterParams.pid      = p_Channel->Audio_pid;
	pesFilterParams.input    = DMX_IN_FRONTEND;
	pesFilterParams.output   = DMX_OUT_TS_TAP;
	pesFilterParams.pes_type = DMX_PES_AUDIO;		// DMX_PES_OTHER  DMX_PES_SUBTITLE
	pesFilterParams.flags    = DMX_IMMEDIATE_START;
	if( ioctl( *p_demux_Audio_pid, DMX_SET_PES_FILTER, &pesFilterParams ) < 0 ) {
			perror( "DEMUX-AUDIO-PID DEVICE" );
			close( *p_demux_Video_pid );
			close( *p_demux_Audio_pid );
			return( FALSE );
	}

	return( TRUE );
}


gboolean StreamTv_get_is_bool_go_tv( void )
{
	return( DataTv.bool_go_tv );
}
void StreamTv_notify_bool_go_tv( gboolean p_bool_go_tv )
{
	DataTv.bool_go_tv = p_bool_go_tv;
}


void StreamTv_write_to_mplayer( gchar *p_str )
{
	write( FD_WRITE, p_str, strlen( p_str ));
}


gpointer *StreamTv_thread( gpointer data )
{
	CHANNEL		*p_Channel = (CHANNEL *)data;
	int 		len;
	struct 		pollfd pfd[1];
	int 		dvr, frontend_fd;
	struct		dvb_frontend_parameters	param;
	int 		demux_Video_pid = -1;
	int 		demux_Audio_pid = -1;

	while( TRUE ) {

		if( DataTv.NumFork > 0 ) {
			// open dvr device
			if( (dvr = open( adapter.dvr, O_RDONLY | O_NONBLOCK )) < 0 ){
				perror("DVR DEVICE");
				pthread_exit(0);
			}

			// set record channel
			if( (frontend_fd = open( adapter.frontend, FE_OPEN_MODE )) < 0 ) {
				close( dvr );
				perror("TUNER DEVICE");
				pthread_exit(0);
			}

			chan_get_param( p_Channel->NameChan, &param );
			ioctl( frontend_fd, FE_SET_FRONTEND, &param );

			// switch video and audio demuxes to dvr
			printf ("Switching dvr on\n");
			if( FALSE == StreamTv_switch_to_record( &demux_Video_pid, &demux_Audio_pid, p_Channel ) ) {
				close( dvr );
				close( frontend_fd );
				pthread_exit(0);
			}
			g_print( "Switching dvr on: Ok\n" );

			pfd[ 0 ].fd     = dvr;
			pfd[ 0 ].events = POLLIN ;

			while( TRUE == StreamTv_get_is_bool_go_tv() ) {
				if( poll( pfd, 1, 2000 )) {
					if( pfd[0].revents & POLLIN ){
						len = read( dvr, DataTv.buf, 1504 );
						if( len < 0 ) {
							perror( "recording" );
							break;
						}
						//
						// FLUX HD
						//
						// Insert PAT/PMT every second second
						// Before paquet buffer [ 1504 ]
						//
						//  me-tv-1.3.1
						// ----------------------
						// line 118 fronted_thread.cc
						// line 133 mpeg_stream.cc
						//
						// pat = stream.build_pat( &pat );
						// write( descripteur_fifo_avi, pat, TS_PACKET_SIZE );		// #define TS_PACKET_SIZE			188
						// pmt = stream.build_pmt( &pmt );
						// write( descripteur_fifo_avi, pmt, TS_PACKET_SIZE );
						// StreamHeader_build_pat( &buf_header[0] );
						// write( descripteur_fifo_avi, buf_header, 188 );
						// StreamHeader_build_pmt( &buf_header[0] );
						// write( descripteur_fifo_avi, buf_header, 188 );
						//
						// if( DataTv.descripteur_fifo_avi > -1 ) {
							write( DataTv.descripteur_fifo_avi, DataTv.buf, len );
						// }
					}
				}
			}
			printf("\n");
			StreamTv_write_to_mplayer( "quit\n" );
			close( dvr );
			close( frontend_fd );
			close( demux_Video_pid );
			close( demux_Audio_pid );
			close( DataTv.descripteur_fifo_avi );
			DataTv.descripteur_fifo_avi = -1;
			// kill( DataTv.NumFork, SIGTERM );
			// sleep( 1 );
			StreamTv_mplayer_remove_fifo();
			close( FD_WRITE );
			close( FD_READ );
			break;
		}
	}
	g_print("END Init stream_tv_thread \n");
	DataTv.NumFork = 0;
	pthread_exit(0);
}

//
//
gboolean fileconv_timeout( gpointer data )
{
	// thread create and quit timeout
	if( DataTv.NumFork == 0 ) {
		gchar		*NameChannel = (gchar *)data;
		CHANNEL		*Channel = NULL;
		gchar		StrToMplayer[ 50 ];
		gchar		StrOutputHandle[ 50 ];

		StreamTv_mplayer_create_fifo();

		if( TRUE == Stream_scan() ) {
			// Args = g_strdup_printf( "mplayer -nojoystick -nolirc -nomouseinput -slave -quiet -input file=%s -idle -wid %d -demuxer lavf -cache 20480 -vf pp=fd", TMP_FIFO, DataTv.outputHandle );
			sprintf( StrOutputHandle, "%d", DataTv.outputHandle );
			StreamTv_make_args(
						"mplayer",
						"-nomouseinput",
						"-nolirc",
						"-slave",
						"-quiet",
						"-idle",
						"-wid",
						StrOutputHandle,
						"-demuxer",
						"lavf",		//  mpegts   lavf
						"-cache",
						"20480",	// 10240 11264 12288 13312 14336 15360 16384 17408 18432 19456 20480
						"-vf",
						"pp=fd",
						NULL
						);
			StreamTv_exec( DataTv.ArgConv );
			StreamTv_remove();
			sprintf( StrToMplayer, "loadfile \"%s\"", DataTv.RepFifo );
			StreamTv_write_to_mplayer( StrToMplayer );
			StreamTv_write_to_mplayer( "get_time_length\n" );

			g_print("BEGIN Init stream_tv_thread \n");
			Stream_print();
			StreamTv_notify_bool_go_tv( TRUE );
			Channel = chan_get_struct( NameChannel );
			g_thread_new( NULL ,(void *)StreamTv_thread, Channel );

		}
		else {
			g_print( "StreamTv_init FAILED !\n" );
			g_print( "    All tuners DVB card are busy !\n" );
			StreamTv_mplayer_remove_fifo();
			wininfo_create (
				WindMain,
				_("All tuners are busy"), // Tous les tuners sont occupés
				  "\n",
				_("   All tuners DVB card are busy  !  "), // Tous les tuners de votre carte DVB sont occupés
				  "\n",
				"");
		}

		// quit timeout
		g_source_remove( DataTv.NumTimeout );
		DataTv.NumTimeout = 0;
	}
	// quit thread
	else {
		StreamTv_notify_bool_go_tv( FALSE );
	}

	return (TRUE);
}


void StreamTv_play( gchar *p_NameChan )
{
	if( DataTv.NumTimeout == 0 )
		DataTv.NumTimeout = g_timeout_add( 50, fileconv_timeout, p_NameChan );
}

gboolean StreamTv_is_playing( void )
{
	return( DataTv.NumFork > 0 ? TRUE : FALSE );
}

/*
http://ftp.chelcom.ru/pub/Drivers/SkyStar/DVB/apps/test/test_tt.c
*/









